Class: Unisec::CLI::Commands::Planes::Reverse

Inherits:
Dry::CLI::Command
  • Object
show all
Defined in:
lib/unisec/cli/planes.rb

Overview

Command unisec planes reverse

Example:

$ unisec planes reverse '…'
Basic Multilingual Plane
$ unisec planes reverse '🨂'
Supplementary Multilingual Plane
$ unisec planes reverse '𠀀'
Supplementary Ideographic Plane
$ unisec planes reverse '🇫🇷'
Supplementary Multilingual Plane

Instance Method Summary collapse

Instance Method Details

#call(char: nil) ⇒ Object

Display the Unicode plane name for a given character

Parameters:

  • char (String) (defaults to: nil)

    Single character (only one code unit, so be careful with emojis, composed or joint characters using several units, only the first code unit will be kept).



121
122
123
# File 'lib/unisec/cli/planes.rb', line 121

def call(char: nil, **)
  Unisec::Planes.reverse_display(char)
end