Class: Unisec::CLI::Commands::Blocks::Reverse
- Inherits:
-
Dry::CLI::Command
- Object
- Dry::CLI::Command
- Unisec::CLI::Commands::Blocks::Reverse
- Defined in:
- lib/unisec/cli/blocks.rb
Overview
Command unisec blocks reverse
Example:
$ unisec blocks reverse '…'
General Punctuation
$ unisec blocks reverse 'A'
Basic Latin
$ unisec blocks reverse '💩'
Miscellaneous Symbols and Pictographs
$ unisec blocks reverse '🇫🇷'
Enclosed Alphanumeric Supplement
Instance Method Summary collapse
-
#call(char: nil) ⇒ Object
Display the Unicode block name for a given character.
Instance Method Details
#call(char: nil) ⇒ Object
Display the Unicode block name for a given character
86 87 88 |
# File 'lib/unisec/cli/blocks.rb', line 86 def call(char: nil, **) Unisec::Blocks.reverse_display(char) end |