Class: Unisec::CLI::Commands::Planes::Reverse
- Inherits:
-
Dry::CLI::Command
- Object
- Dry::CLI::Command
- Unisec::CLI::Commands::Planes::Reverse
- Defined in:
- lib/unisec/cli/planes.rb
Overview
Command unisec planes reverse
Example:
$ unisec planes reverse '…'
Basic Multilingual Plane
$ unisec planes reverse '🨂'
Supplementary Multilingual Plane
$ unisec planes reverse '𠀀'
Supplementary Ideographic Plane
$ unisec planes reverse '🇫🇷'
Supplementary Multilingual Plane
Instance Method Summary collapse
-
#call(char: nil) ⇒ Object
Display the Unicode plane name for a given character.
Instance Method Details
#call(char: nil) ⇒ Object
Display the Unicode plane name for a given character
121 122 123 |
# File 'lib/unisec/cli/planes.rb', line 121 def call(char: nil, **) Unisec::Planes.reverse_display(char) end |